@@ -24,6 +24,8 @@
 #include <malloc.h>
 #include <xmlb.h>
 
+#include "config.h"
+
 #include "bz-backend-notification.h"
 #include "bz-backend-transaction-op-payload.h"
 #include "bz-backend-transaction-op-progress-payload.h"
@@ -58,6 +60,10 @@ struct _BzFlatpakInstance
   GMutex     notif_mutex;
   GPtrArray *notif_channels;
   DexFuture *notif_send;
+
+  GMutex transactions_mutex;
+  /* BzEntry* -> GPtrArray* -> GCancellable* */
+  GHashTable *ongoing_cancellables;
 };
 
 static void
@@ -342,6 +348,9 @@ bz_flatpak_instance_dispose (GObject *object)
   dex_clear (&self->notif_send);
   g_mutex_clear (&self->notif_mutex);
 
+  g_clear_pointer (&self->ongoing_cancellables, g_hash_table_unref);
+  g_mutex_clear (&self->transactions_mutex);
+
   G_OBJECT_CLASS (bz_flatpak_instance_parent_class)->dispose (object);
 }
 
@@ -359,9 +368,15 @@ bz_flatpak_instance_init (BzFlatpakInstance *self)
   self->scheduler   = dex_thread_pool_scheduler_new ();
   self->system_mute = 0;
   self->user_mute   = 0;
+
   g_mutex_init (&self->mute_mutex);
+
   self->notif_channels = g_ptr_array_new_with_free_func (dex_unref);
   g_mutex_init (&self->notif_mutex);
+
+  self->ongoing_cancellables = g_hash_table_new_full (
+      g_direct_hash, g_direct_equal, g_object_unref, (GDestroyNotify) g_ptr_array_unref);
+  g_mutex_init (&self->transactions_mutex);
 }
 
 static DexChannel *
@@ -540,6 +555,31 @@ bz_flatpak_instance_schedule_transaction (BzBackend    *backend,
       transaction_data_unref);
 }
 
+static gboolean
+bz_flatpak_instance_cancel_task_for_entry (BzBackend *backend,
+                                           BzEntry   *entry)
+{
+  BzFlatpakInstance *self         = BZ_FLATPAK_INSTANCE (backend);
+  g_autoptr (GMutexLocker) locker = NULL;
+  GPtrArray *cancellables         = NULL;
+
+  locker = g_mutex_locker_new (&self->transactions_mutex);
+
+  cancellables = g_hash_table_lookup (self->ongoing_cancellables, entry);
+  if (cancellables == NULL)
+    return FALSE;
+
+  for (guint i = 0; i < cancellables->len; i++)
+    {
+      GCancellable *cancellable = NULL;
+
+      cancellable = g_ptr_array_index (cancellables, i);
+      g_cancellable_cancel (cancellable);
+    }
+
+  return TRUE;
+}
+
 static void
 backend_iface_init (BzBackendInterface *iface)
 {
@@ -550,6 +590,7 @@ backend_iface_init (BzBackendInterface *iface)
   iface->retrieve_update_ids         = bz_flatpak_instance_retrieve_update_ids;
   iface->list_repositories           = bz_flatpak_instance_list_repositories;
   iface->schedule_transaction        = bz_flatpak_instance_schedule_transaction;
+  iface->cancel_task_for_entry       = bz_flatpak_instance_cancel_task_for_entry;
 }
 
 FlatpakInstallation *
@@ -653,7 +694,23 @@ init_fiber (InitData *data)
       g_clear_pointer (&local_error, g_error_free);
     }
 
+#ifdef SANDBOXED_LIBFLATPAK
+  {
+    g_autoptr (GFile) user_installation_path = NULL;
+    const char      *home                    = g_get_home_dir ();
+    g_autofree char *user_flatpak_path       = g_build_filename (home, ".local", "share", "flatpak", NULL);
+
+    user_installation_path = g_file_new_for_path (user_flatpak_path);
+    self->user             = flatpak_installation_new_for_path (
+        user_installation_path,
+        TRUE,
+        NULL,
+        &local_error);
+  }
+#else
   self->user = flatpak_installation_new_user (NULL, &local_error);
+#endif
+
   if (self->user != NULL)
     {
       self->user_events = flatpak_installation_create_monitor (
@@ -1048,13 +1105,6 @@ retrieve_remote_refs_fiber (GatherRefsData *data)
 
       name = flatpak_remote_get_name (remote);
 
-      if (strstr (name, "fedora") != NULL)
-        {
-          g_debug ("Skipping remote %s", name);
-          /* the fedora flatpak repos cause too many issues */
-          continue;
-        }
-
       job_data               = retrieve_refs_for_remote_data_new ();
       job_data->parent       = gather_refs_data_ref (data);
       job_data->installation = g_object_ref (installation);
@@ -1477,11 +1527,22 @@ retrieve_refs_for_remote_fiber (RetrieveRefsForRemoteData *data)
   FlatpakRemote       *remote         = data->remote;
   const char          *remote_name    = NULL;
   gboolean             is_noenumerate = FALSE;
+  g_autoptr (BzFlatpakInstance) self  = NULL;
+
+  bz_weak_get_or_return_reject (self, data->parent->self);
 
   remote_name    = flatpak_remote_get_name (remote);
   is_noenumerate = flatpak_remote_get_noenumerate (remote);
 
+  /* the fedora flatpak repos cause too many issues */
+  if (strstr (remote_name, "fedora") != NULL)
+    is_noenumerate = TRUE;
+
+#ifdef SANDBOXED_LIBFLATPAK
+  if (is_noenumerate || installation == self->user)
+#else
   if (is_noenumerate)
+#endif
     return retrieve_refs_for_noenumerable_remote (data, remote_name, installation, remote);
   else
     return retrieve_refs_for_enumerable_remote (data, remote_name, installation, remote);
@@ -1531,12 +1592,14 @@ retrieve_installs_fiber (GatherRefsData *data)
       n_user_refs = user_refs->len;
     }
 
-  ids = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+  ids = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 
   for (guint i = 0; i < n_system_refs + n_user_refs; i++)
     {
-      gboolean             user = FALSE;
-      FlatpakInstalledRef *iref = NULL;
+      gboolean             user      = FALSE;
+      FlatpakInstalledRef *iref      = NULL;
+      const char          *version   = NULL;
+      g_autofree char     *unique_id = NULL;
 
       if (i < n_system_refs)
         {
@@ -1549,13 +1612,30 @@ retrieve_installs_fiber (GatherRefsData *data)
           iref = g_ptr_array_index (user_refs, i - n_system_refs);
         }
 
-      g_hash_table_replace (ids, bz_flatpak_ref_format_unique (FLATPAK_REF (iref), user), NULL);
+      version = flatpak_installed_ref_get_appdata_version (iref);
+
+      unique_id = bz_flatpak_ref_format_unique (FLATPAK_REF (iref), user);
+
+      g_hash_table_replace (ids,
+                            g_steal_pointer (&unique_id),
+                            g_strdup (version != NULL ? version : ""));
     }
 
   return dex_future_new_take_boxed (
       G_TYPE_HASH_TABLE, g_steal_pointer (&ids));
 }
 
+static gboolean
+should_skip_extension_ref (FlatpakInstalledRef *iref)
+{
+  const gchar *ref_name = flatpak_ref_get_name (FLATPAK_REF (iref));
+
+  /* These get updated with their parents and look really bad in the UI */
+  return g_str_has_suffix (ref_name, ".Locale") ||
+         g_str_has_suffix (ref_name, ".Debug") ||
+         g_str_has_suffix (ref_name, ".Sources");
+}
+
 static DexFuture *
 retrieve_updates_fiber (GatherRefsData *data)
 {
@@ -1597,7 +1677,6 @@ retrieve_updates_fiber (GatherRefsData *data)
     }
 
   ids = g_ptr_array_new_with_free_func (g_free);
-  g_ptr_array_set_size (ids, n_sys_refs + n_user_refs);
 
   for (guint i = 0; i < n_sys_refs + n_user_refs; i++)
     {
@@ -1615,8 +1694,11 @@ retrieve_updates_fiber (GatherRefsData *data)
           iref = g_ptr_array_index (user_refs, i - n_sys_refs);
         }
 
-      g_ptr_array_index (ids, i) =
-          bz_flatpak_ref_format_unique (FLATPAK_REF (iref), user);
+      if (should_skip_extension_ref (iref))
+        continue;
+
+      g_ptr_array_add (ids,
+                       bz_flatpak_ref_format_unique (FLATPAK_REF (iref), user));
     }
 
   return dex_future_new_take_boxed (
@@ -1935,6 +2017,59 @@ transaction_fiber (TransactionData *data)
         }
     }
 
+  g_mutex_lock (&self->transactions_mutex);
+
+#define REGISTER_CANCELLABLES(entry)                                                           \
+  G_STMT_START                                                                                 \
+  {                                                                                            \
+    GPtrArray *cancellables = NULL;                                                            \
+                                                                                               \
+    cancellables = g_hash_table_lookup (self->ongoing_cancellables, entry);                    \
+    if (cancellables != NULL)                                                                  \
+      g_ptr_array_add (cancellables, g_object_ref (cancellable));                              \
+    else                                                                                       \
+      {                                                                                        \
+        cancellables = g_ptr_array_new_with_free_func (g_object_unref);                        \
+        g_ptr_array_add (cancellables, g_object_ref (cancellable));                            \
+        g_hash_table_replace (self->ongoing_cancellables, g_object_ref (entry), cancellables); \
+      }                                                                                        \
+  }                                                                                            \
+  G_STMT_END
+
+  if (installations != NULL)
+    {
+      for (guint i = 0; i < installations->len; i++)
+        {
+          BzEntry *entry = NULL;
+
+          entry = g_ptr_array_index (installations, i);
+          REGISTER_CANCELLABLES (entry);
+        }
+    }
+  if (removals != NULL)
+    {
+      for (guint i = 0; i < removals->len; i++)
+        {
+          BzEntry *entry = NULL;
+
+          entry = g_ptr_array_index (removals, i);
+          REGISTER_CANCELLABLES (entry);
+        }
+    }
+  if (updates != NULL)
+    {
+      for (guint i = 0; i < updates->len; i++)
+        {
+          BzEntry *entry = NULL;
+
+          entry = g_ptr_array_index (updates, i);
+          REGISTER_CANCELLABLES (entry);
+        }
+    }
+
+#undef REGISTER_CANCELLABLES
+  g_mutex_unlock (&self->transactions_mutex);
+
   jobs = g_ptr_array_new_with_free_func (dex_unref);
   for (guint i = 0; i < transactions->len; i++)
     {
@@ -1961,6 +2096,58 @@ transaction_fiber (TransactionData *data)
                  (DexFuture *const *) jobs->pdata,
                  jobs->len),
              NULL);
+
+  g_mutex_lock (&self->transactions_mutex);
+
+#define UNREGISTER_CANCELLABLES(entry)                                      \
+  G_STMT_START                                                              \
+  {                                                                         \
+    GPtrArray *cancellables = NULL;                                         \
+                                                                            \
+    cancellables = g_hash_table_lookup (self->ongoing_cancellables, entry); \
+    if (cancellables != NULL)                                               \
+      {                                                                     \
+        g_ptr_array_remove (cancellables, cancellable);                     \
+        if (cancellables->len == 0)                                         \
+          g_hash_table_remove (self->ongoing_cancellables, entry);          \
+      }                                                                     \
+  }                                                                         \
+  G_STMT_END
+
+  if (installations != NULL)
+    {
+      for (guint i = 0; i < installations->len; i++)
+        {
+          BzEntry *entry = NULL;
+
+          entry = g_ptr_array_index (installations, i);
+          UNREGISTER_CANCELLABLES (entry);
+        }
+    }
+  if (removals != NULL)
+    {
+      for (guint i = 0; i < removals->len; i++)
+        {
+          BzEntry *entry = NULL;
+
+          entry = g_ptr_array_index (removals, i);
+          UNREGISTER_CANCELLABLES (entry);
+        }
+    }
+  if (updates != NULL)
+    {
+      for (guint i = 0; i < updates->len; i++)
+        {
+          BzEntry *entry = NULL;
+
+          entry = g_ptr_array_index (updates, i);
+          UNREGISTER_CANCELLABLES (entry);
+        }
+    }
+
+#undef UNREGISTER_CANCELLABLES
+  g_mutex_unlock (&self->transactions_mutex);
+
   if (data->send_futures->len > 0)
     dex_await (dex_future_allv (
                    (DexFuture *const *) data->send_futures->pdata,
@@ -2103,6 +2290,11 @@ transaction_operation_done (FlatpakTransaction          *transaction,
   gboolean                        is_user           = FALSE;
   g_autofree char                *unique_id         = NULL;
   g_autoptr (BzBackendNotification) notif           = NULL;
+  const char          *version                      = NULL;
+  FlatpakInstallation *installation                 = NULL;
+  g_autoptr (FlatpakInstalledRef) iref              = NULL;
+  g_autoptr (GError) local_error                    = NULL;
+  g_autoptr (FlatpakRef) parsed_ref                 = NULL;
 
   bz_weak_get_or_return (self, data->self);
 
@@ -2147,9 +2339,45 @@ transaction_operation_done (FlatpakTransaction          *transaction,
   is_user   = flatpak_transaction_get_installation (transaction) == self->user;
   unique_id = bz_flatpak_ref_parts_format_unique (origin, ref, is_user);
 
+  if (notif_kind == BZ_BACKEND_NOTIFICATION_KIND_INSTALL_DONE ||
+      notif_kind == BZ_BACKEND_NOTIFICATION_KIND_UPDATE_DONE)
+    {
+      installation = flatpak_transaction_get_installation (transaction);
+
+      parsed_ref = flatpak_ref_parse (ref, &local_error);
+      if (parsed_ref != NULL)
+        {
+          iref = flatpak_installation_get_installed_ref (
+              installation,
+              flatpak_ref_get_kind (parsed_ref),
+              flatpak_ref_get_name (parsed_ref),
+              flatpak_ref_get_arch (parsed_ref),
+              flatpak_ref_get_branch (parsed_ref),
+              NULL,
+              &local_error);
+
+          if (iref != NULL)
+            version = flatpak_installed_ref_get_appdata_version (iref);
+          else if (local_error != NULL)
+            {
+              g_warning ("Failed to get installed ref for version: %s", local_error->message);
+              g_clear_error (&local_error);
+            }
+        }
+      else if (local_error != NULL)
+        {
+          g_warning ("Failed to parse ref for version: %s", local_error->message);
+          g_clear_error (&local_error);
+        }
+    }
+
   notif = bz_backend_notification_new ();
   bz_backend_notification_set_kind (notif, notif_kind);
   bz_backend_notification_set_unique_id (notif, unique_id);
+
+  if (version != NULL && *version != '\0')
+    bz_backend_notification_set_version (notif, version);
+
   send_notif_all (self, notif, TRUE);
 }
 
