@@ -38,7 +38,6 @@
 #include "bz-flatpak-entry.h"
 #include "bz-full-view.h"
 #include "bz-hardware-support-dialog.h"
-#include "bz-lazy-async-texture-model.h"
 #include "bz-license-dialog.h"
 #include "bz-releases-list.h"
 #include "bz-safety-calculator.h"
@@ -52,6 +51,7 @@
 #include "bz-stats-dialog.h"
 #include "bz-tag-list.h"
 #include "bz-template-callbacks.h"
+#include "bz-util.h"
 
 struct _BzFullView
 {
@@ -60,7 +60,9 @@ struct _BzFullView
   BzStateInfo          *state;
   BzTransactionManager *transactions;
   BzEntryGroup         *group;
+  DexFuture            *ui_future;
   BzResult             *ui_entry;
+  BzResult             *runtime;
   BzResult             *group_model;
   gboolean              show_sidebar;
 
@@ -117,10 +119,12 @@ bz_full_view_dispose (GObject *object)
 {
   BzFullView *self = BZ_FULL_VIEW (object);
 
+  dex_clear (&self->ui_future);
   g_clear_object (&self->state);
   g_clear_object (&self->transactions);
   g_clear_object (&self->group);
   g_clear_object (&self->ui_entry);
+  g_clear_object (&self->runtime);
   g_clear_object (&self->group_model);
   g_clear_object (&self->main_menu);
 
@@ -171,12 +175,11 @@ bz_full_view_set_property (GObject      *object,
     case PROP_ENTRY_GROUP:
       bz_full_view_set_entry_group (self, g_value_get_object (value));
       break;
-    case PROP_UI_ENTRY:
     case PROP_MAIN_MENU:
-      if (self->main_menu)
-        g_object_unref (self->main_menu);
+      g_clear_object (&self->main_menu);
       self->main_menu = g_value_dup_object (value);
       break;
+    case PROP_UI_ENTRY:
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -200,6 +203,13 @@ format_with_small_suffix (char *number, const char *suffix)
                           number, suffix);
 }
 
+static gboolean
+is_scrolled_down (gpointer object,
+                  double   value)
+{
+  return value > 100.0;
+}
+
 static char *
 format_favorites_count (gpointer object,
                         int      favorites_count)
@@ -274,9 +284,16 @@ format_size (gpointer object, guint64 value)
 
 static char *
 get_size_label (gpointer object,
-                gboolean is_installable)
+                gboolean is_installable,
+                gboolean runtime_installed,
+                guint64  runtime_size)
 {
-  // Translators: .
+  if (is_installable && !runtime_installed && runtime_size > 0)
+    {
+      g_autofree char *size_str = g_format_size (runtime_size);
+      return g_strdup_printf (_ ("+%s runtime"), size_str);
+    }
+
   return g_strdup (is_installable ? _ ("Download") : _ ("Installed"));
 }
 
@@ -940,7 +957,7 @@ run_cb (BzFullView *self,
 
               window = gtk_widget_get_ancestor (GTK_WIDGET (button), GTK_TYPE_WINDOW);
               if (window != NULL)
-                bz_show_error_for_widget (window, local_error->message);
+                bz_show_error_for_widget (window, _ ("Failed to launch application"), local_error->message);
             }
           break;
         }
@@ -1158,7 +1175,6 @@ bz_full_view_class_init (BzFullViewClass *klass)
   g_type_ensure (BZ_TYPE_FAVORITE_BUTTON);
   g_type_ensure (BZ_TYPE_FLATPAK_ENTRY);
   g_type_ensure (BZ_TYPE_HARDWARE_SUPPORT_DIALOG);
-  g_type_ensure (BZ_TYPE_LAZY_ASYNC_TEXTURE_MODEL);
   g_type_ensure (BZ_TYPE_SECTION_VIEW);
   g_type_ensure (BZ_TYPE_RELEASES_LIST);
   g_type_ensure (BZ_TYPE_SCREENSHOTS_CAROUSEL);
@@ -1177,6 +1193,7 @@ bz_full_view_class_init (BzFullViewClass *klass)
   gtk_widget_class_bind_template_child (widget_class, BzFullView, wide_install_button);
   gtk_widget_class_bind_template_child (widget_class, BzFullView, narrow_install_button);
   gtk_widget_class_bind_template_child (widget_class, BzFullView, narrow_open_button);
+  gtk_widget_class_bind_template_callback (widget_class, is_scrolled_down);
   gtk_widget_class_bind_template_callback (widget_class, format_favorites_count);
   gtk_widget_class_bind_template_callback (widget_class, format_recent_downloads);
   gtk_widget_class_bind_template_callback (widget_class, format_recent_downloads_tooltip);
@@ -1239,6 +1256,29 @@ bz_full_view_new (void)
   return g_object_new (BZ_TYPE_FULL_VIEW, NULL);
 }
 
+static DexFuture *
+on_ui_entry_resolved (DexFuture *future,
+                      GWeakRef  *wr)
+{
+  g_autoptr (BzFullView) self         = NULL;
+  BzEntry *ui_entry                   = NULL;
+  g_autoptr (BzResult) runtime_result = NULL;
+  const GValue *value                 = NULL;
+
+  bz_weak_get_or_return_reject (self, wr);
+
+  value = dex_future_get_value (future, NULL);
+  if (value != NULL && G_VALUE_HOLDS_OBJECT (value))
+    {
+      ui_entry = g_value_get_object (value);
+
+      if (BZ_IS_FLATPAK_ENTRY (ui_entry))
+        self->runtime = bz_flatpak_entry_dup_runtime_result (BZ_FLATPAK_ENTRY (ui_entry));
+    }
+
+  return dex_future_new_for_boolean (TRUE);
+}
+
 void
 bz_full_view_set_entry_group (BzFullView   *self,
                               BzEntryGroup *group)
@@ -1250,8 +1290,10 @@ bz_full_view_set_entry_group (BzFullView   *self,
   if (group == self->group)
     return;
 
+  dex_clear (&self->ui_future);
   g_clear_object (&self->group);
   g_clear_object (&self->ui_entry);
+  g_clear_object (&self->runtime);
   g_clear_object (&self->group_model);
   gtk_toggle_button_set_active (self->description_toggle, FALSE);
 
@@ -1262,9 +1304,11 @@ bz_full_view_set_entry_group (BzFullView   *self,
 
       if (self->ui_entry != NULL && bz_result_get_resolved (self->ui_entry))
         {
-          g_autoptr (GListStore) store = NULL;
-          g_autoptr (DexFuture) future = NULL;
-          BzEntry *entry               = NULL;
+          BzEntry *entry                      = NULL;
+          g_autoptr (GListStore) store        = NULL;
+          g_autoptr (DexFuture) future        = NULL;
+          g_autoptr (DexFuture) object_future = NULL;
+          GWeakRef *wr                        = NULL;
 
           entry = bz_result_get_object (self->ui_entry);
           store = g_list_store_new (BZ_TYPE_ENTRY);
@@ -1272,6 +1316,11 @@ bz_full_view_set_entry_group (BzFullView   *self,
 
           future            = dex_future_new_for_object (store);
           self->group_model = bz_result_new (future);
+
+          object_future = dex_future_new_for_object (entry);
+          wr            = bz_track_weak (self);
+          dex_unref (on_ui_entry_resolved (object_future, wr));
+          bz_weak_release (wr);
         }
       else
         {
@@ -1279,6 +1328,19 @@ bz_full_view_set_entry_group (BzFullView   *self,
 
           future            = bz_entry_group_dup_all_into_store (group);
           self->group_model = bz_result_new (future);
+
+          if (self->ui_entry != NULL)
+            {
+              g_autoptr (DexFuture) ui_future = NULL;
+
+              ui_future = bz_result_dup_future (self->ui_entry);
+              ui_future = dex_future_then (
+                  ui_future,
+                  (DexFutureCallback) on_ui_entry_resolved,
+                  bz_track_weak (self),
+                  bz_weak_release);
+              self->ui_future = g_steal_pointer (&ui_future);
+            }
         }
 
       adw_view_stack_set_visible_child_name (self->stack, "content");
