@@ -33,13 +33,14 @@
 #include "bz-flathub-page.h"
 #include "bz-full-view.h"
 #include "bz-global-progress.h"
-#include "bz-installed-page.h"
+#include "bz-hooks.h"
 #include "bz-io.h"
+#include "bz-library-page.h"
+#include "bz-popup-overlay.h"
 #include "bz-progress-bar.h"
 #include "bz-search-widget.h"
 #include "bz-template-callbacks.h"
 #include "bz-transaction-dialog.h"
-#include "bz-transaction-list-dialog.h"
 #include "bz-transaction-manager.h"
 #include "bz-user-data-page.h"
 #include "bz-util.h"
@@ -56,23 +57,16 @@ struct _BzWindow
   gboolean breakpoint_applied;
 
   /* Template widgets */
-  BzCometOverlay      *comet_overlay;
-  AdwOverlaySplitView *split_view;
-  AdwViewStack        *transactions_stack;
-  AdwNavigationView   *navigation_view;
-  BzFullView          *full_view;
-  GtkToggleButton     *toggle_transactions;
-  GtkToggleButton     *toggle_transactions_sidebar;
-  BzSearchWidget      *search_widget;
-  BzInstalledPage     *installed_page;
-  GtkButton           *update_button;
-  GtkToggleButton     *transactions_pause;
-  GtkButton           *transactions_stop;
-  GtkButton           *transactions_clear;
-  AdwToastOverlay     *toasts;
-  AdwViewStack        *main_view_stack;
-  GtkStack            *main_stack;
-  GtkLabel            *debug_id_label;
+  BzCometOverlay    *comet_overlay;
+  BzPopupOverlay    *popup_overlay;
+  AdwNavigationView *navigation_view;
+  BzFullView        *full_view;
+  BzSearchWidget    *search_widget;
+  BzLibraryPage     *library_page;
+  AdwToastOverlay   *toasts;
+  AdwViewStack      *main_view_stack;
+  GtkStack          *main_stack;
+  GtkLabel          *debug_id_label;
 };
 
 G_DEFINE_FINAL_TYPE (BzWindow, bz_window, ADW_TYPE_APPLICATION_WINDOW)
@@ -119,11 +113,6 @@ BZ_DEFINE_DATA (
 static DexFuture *
 bulk_install_fiber (BulkInstallData *data);
 
-static void
-update_dialog_response (BzTransactionListDialog *dialog,
-                        const char              *response,
-                        BzWindow                *self);
-
 static DexFuture *
 transact (BzWindow  *self,
           BzEntry   *entry,
@@ -138,11 +127,6 @@ try_transact (BzWindow     *self,
               gboolean      auto_confirm,
               GtkWidget    *source);
 
-static void
-update (BzWindow *self,
-        BzEntry **updates,
-        guint     n_updates);
-
 static void
 search (BzWindow   *self,
         const char *text);
@@ -152,9 +136,6 @@ bulk_install (BzWindow *self,
               BzEntry **installs,
               guint     n_installs);
 
-static void
-check_transactions (BzWindow *self);
-
 static void
 set_page (BzWindow *self);
 
@@ -213,9 +194,8 @@ list_length (gpointer    object,
 }
 
 static void
-browser_group_selected_cb (BzWindow     *self,
-                           BzEntryGroup *group,
-                           gpointer      browser)
+select_cb (BzWindow     *self,
+           BzEntryGroup *group)
 {
   bz_window_show_group (self, group);
 }
@@ -280,19 +260,89 @@ remove_addon_cb (BzWindow   *self,
 }
 
 static void
-install_entry_cb (BzWindow   *self,
-                  BzEntry    *entry,
-                  BzFullView *view)
+install_entry_cb (BzWindow     *self,
+                  BzEntryGroup *group,
+                  BzFullView   *view)
 {
-  try_transact (self, entry, NULL, FALSE, FALSE, NULL);
+  try_transact (self, NULL, group, FALSE, FALSE, NULL);
 }
 
 static void
-remove_installed_cb (BzWindow   *self,
-                     BzEntry    *entry,
-                     BzFullView *view)
+remove_installed_cb (BzWindow     *self,
+                     BzEntryGroup *group,
+                     BzFullView   *view)
 {
-  try_transact (self, entry, NULL, TRUE, FALSE, NULL);
+  try_transact (self, NULL, group, TRUE, FALSE, NULL);
+}
+
+static void
+update_cb (BzWindow      *self,
+           GListModel    *entries,
+           BzLibraryPage *library_page)
+{
+  g_autoptr (BzTransaction) transaction  = NULL;
+  guint                n_updates         = 0;
+  g_autofree BzEntry **updates_buf       = NULL;
+  GListModel          *available_updates = NULL;
+
+  g_return_if_fail (BZ_IS_WINDOW (self));
+  g_return_if_fail (G_IS_LIST_MODEL (entries));
+  g_return_if_fail (BZ_IS_LIBRARY_PAGE (library_page));
+
+  n_updates = g_list_model_get_n_items (entries);
+  if (n_updates == 0)
+    return;
+
+  updates_buf = g_malloc_n (n_updates, sizeof (*updates_buf));
+  for (guint i = 0; i < n_updates; i++)
+    updates_buf[i] = g_list_model_get_item (entries, i);
+
+  transaction = bz_transaction_new_full (
+      NULL, 0,
+      updates_buf, n_updates,
+      NULL, 0);
+
+  dex_future_disown (bz_transaction_manager_add (
+      bz_state_info_get_transaction_manager (self->state),
+      transaction));
+
+  available_updates = bz_state_info_get_available_updates (self->state);
+  if (G_IS_LIST_STORE (available_updates))
+    {
+      GListStore *store       = G_LIST_STORE (available_updates);
+      guint       n_available = g_list_model_get_n_items (available_updates);
+
+      for (guint i = n_available; i > 0; i--)
+        {
+          guint current_size                  = 0;
+          guint idx                           = 0;
+          g_autoptr (BzEntry) available_entry = NULL;
+          const char *available_id            = NULL;
+
+          idx          = i - 1;
+          current_size = g_list_model_get_n_items (available_updates);
+
+          if (idx >= current_size)
+            continue;
+
+          available_entry = g_list_model_get_item (available_updates, idx);
+          available_id    = bz_entry_get_id (available_entry);
+
+          for (guint j = 0; j < n_updates; j++)
+            {
+              if (g_strcmp0 (available_id, bz_entry_get_id (updates_buf[j])) == 0)
+                {
+                  g_list_store_remove (store, idx);
+                  break;
+                }
+            }
+        }
+    }
+
+  g_object_notify (G_OBJECT (self->state), "available-updates");
+
+  for (guint i = 0; i < n_updates; i++)
+    g_object_unref (updates_buf[i]);
 }
 
 static void
@@ -300,36 +350,7 @@ bulk_install_cb (BzWindow   *self,
                  GListModel *groups,
                  gpointer    source)
 {
-  g_autoptr (BulkInstallData) data = NULL;
-
-  g_return_if_fail (BZ_IS_WINDOW (self));
-  g_return_if_fail (G_IS_LIST_MODEL (groups));
-
-  data         = bulk_install_data_new ();
-  data->self   = bz_track_weak (self);
-  data->groups = g_object_ref (groups);
-
-  dex_future_disown (dex_scheduler_spawn (
-      dex_scheduler_get_default (),
-      bz_get_dex_stack_size (),
-      (DexFiberFunc) bulk_install_fiber,
-      bulk_install_data_ref (data),
-      bulk_install_data_unref));
-}
-
-static void
-installed_page_show_cb (BzWindow   *self,
-                        BzEntry    *entry,
-                        BzFullView *view)
-{
-  g_autoptr (BzEntryGroup) group = NULL;
-
-  group = bz_application_map_factory_convert_one (
-      bz_state_info_get_application_factory (self->state),
-      gtk_string_object_new (bz_entry_get_id (entry)));
-
-  if (group != NULL)
-    bz_window_show_group (self, group);
+  bz_window_bulk_install (self, groups);
 }
 
 void
@@ -389,26 +410,6 @@ breakpoint_unapply_cb (BzWindow      *self,
   gtk_widget_remove_css_class (GTK_WIDGET (self), "narrow");
 }
 
-static void
-pause_transactions_cb (BzWindow        *self,
-                       GtkToggleButton *toggle)
-{
-  gboolean paused = FALSE;
-
-  paused = gtk_toggle_button_get_active (toggle);
-  bz_transaction_manager_set_paused (
-      bz_state_info_get_transaction_manager (self->state), paused);
-  check_transactions (self);
-}
-
-static void
-stop_transactions_cb (BzWindow  *self,
-                      GtkButton *button)
-{
-  bz_transaction_manager_set_paused (bz_state_info_get_transaction_manager (self->state), TRUE);
-  bz_transaction_manager_cancel_current (bz_state_info_get_transaction_manager (self->state));
-}
-
 static void
 sync_cb (BzWindow  *self,
          GtkButton *button)
@@ -418,31 +419,6 @@ sync_cb (BzWindow  *self,
       "sync-remotes", NULL);
 }
 
-static void
-update_cb (BzWindow  *self,
-           GtkButton *button)
-{
-  /* if the button is clickable, there have to be updates */
-  bz_window_push_update_dialog (self);
-}
-
-static char *
-update_amount_tooltip (gpointer    object,
-                       GListModel *model)
-{
-  guint count;
-
-  if (model == NULL)
-    count = 0;
-  else
-    count = g_list_model_get_n_items (model);
-
-  return g_strdup_printf (ngettext ("%d Update Available",
-                                    "%d Updates Available",
-                                    count),
-                          count);
-}
-
 static void
 transactions_clear_cb (BzWindow  *self,
                        GtkButton *button)
@@ -465,10 +441,7 @@ action_escape (GtkWidget  *widget,
 
   adw_navigation_view_pop (self->navigation_view);
   if (n_pages <= 2)
-    {
-      gtk_toggle_button_set_active (self->toggle_transactions, FALSE);
-      set_page (self);
-    }
+    set_page (self);
 }
 
 static char *
@@ -490,6 +463,18 @@ action_user_data (GtkWidget  *widget,
   adw_navigation_view_push (self->navigation_view, user_data_page);
 }
 
+static void
+action_open_library (GtkWidget  *widget,
+                     const char *action_name,
+                     GVariant   *parameter)
+{
+  BzWindow *self = BZ_WINDOW (widget);
+
+  adw_navigation_view_pop_to_tag (self->navigation_view, "main");
+  adw_view_stack_set_visible_child_name (self->main_view_stack, "installed");
+  bz_library_page_reset_search (self->library_page);
+}
+
 static void
 debug_id_inspect_cb (BzWindow  *self,
                      GtkButton *button)
@@ -538,51 +523,42 @@ bz_window_class_init (BzWindowClass *klass)
   g_object_class_install_properties (object_class, LAST_PROP, props);
 
   g_type_ensure (BZ_TYPE_COMET_OVERLAY);
+  g_type_ensure (BZ_TYPE_POPUP_OVERLAY);
   g_type_ensure (BZ_TYPE_SEARCH_WIDGET);
   g_type_ensure (BZ_TYPE_GLOBAL_PROGRESS);
   g_type_ensure (BZ_TYPE_PROGRESS_BAR);
   g_type_ensure (BZ_TYPE_CURATED_VIEW);
   g_type_ensure (BZ_TYPE_FULL_VIEW);
-  g_type_ensure (BZ_TYPE_INSTALLED_PAGE);
+  g_type_ensure (BZ_TYPE_LIBRARY_PAGE);
   g_type_ensure (BZ_TYPE_FLATHUB_PAGE);
+  // g_type_ensure (BZ_TYPE_VIEW_SWITCHER);
 
   gtk_widget_class_set_template_from_resource (widget_class, "/io/github/kolunmi/Bazaar/bz-window.ui");
   bz_widget_class_bind_all_util_callbacks (widget_class);
 
   gtk_widget_class_bind_template_child (widget_class, BzWindow, comet_overlay);
-  gtk_widget_class_bind_template_child (widget_class, BzWindow, split_view);
-  gtk_widget_class_bind_template_child (widget_class, BzWindow, transactions_stack);
+  gtk_widget_class_bind_template_child (widget_class, BzWindow, popup_overlay);
   gtk_widget_class_bind_template_child (widget_class, BzWindow, navigation_view);
   gtk_widget_class_bind_template_child (widget_class, BzWindow, full_view);
   gtk_widget_class_bind_template_child (widget_class, BzWindow, toasts);
-  gtk_widget_class_bind_template_child (widget_class, BzWindow, toggle_transactions);
-  gtk_widget_class_bind_template_child (widget_class, BzWindow, toggle_transactions_sidebar);
   gtk_widget_class_bind_template_child (widget_class, BzWindow, search_widget);
-  gtk_widget_class_bind_template_child (widget_class, BzWindow, installed_page);
-  gtk_widget_class_bind_template_child (widget_class, BzWindow, update_button);
-  gtk_widget_class_bind_template_child (widget_class, BzWindow, transactions_pause);
-  gtk_widget_class_bind_template_child (widget_class, BzWindow, transactions_stop);
-  gtk_widget_class_bind_template_child (widget_class, BzWindow, transactions_clear);
+  gtk_widget_class_bind_template_child (widget_class, BzWindow, library_page);
   gtk_widget_class_bind_template_child (widget_class, BzWindow, main_view_stack);
   gtk_widget_class_bind_template_child (widget_class, BzWindow, main_stack);
   gtk_widget_class_bind_template_child (widget_class, BzWindow, debug_id_label);
   gtk_widget_class_bind_template_callback (widget_class, list_length);
-  gtk_widget_class_bind_template_callback (widget_class, browser_group_selected_cb);
+  gtk_widget_class_bind_template_callback (widget_class, select_cb);
   gtk_widget_class_bind_template_callback (widget_class, search_widget_select_cb);
   gtk_widget_class_bind_template_callback (widget_class, full_view_install_cb);
   gtk_widget_class_bind_template_callback (widget_class, full_view_remove_cb);
   gtk_widget_class_bind_template_callback (widget_class, install_addon_cb);
   gtk_widget_class_bind_template_callback (widget_class, remove_addon_cb);
   gtk_widget_class_bind_template_callback (widget_class, remove_installed_cb);
-  gtk_widget_class_bind_template_callback (widget_class, installed_page_show_cb);
+  gtk_widget_class_bind_template_callback (widget_class, update_cb);
   gtk_widget_class_bind_template_callback (widget_class, page_toggled_cb);
   gtk_widget_class_bind_template_callback (widget_class, breakpoint_apply_cb);
   gtk_widget_class_bind_template_callback (widget_class, breakpoint_unapply_cb);
-  gtk_widget_class_bind_template_callback (widget_class, pause_transactions_cb);
-  gtk_widget_class_bind_template_callback (widget_class, stop_transactions_cb);
   gtk_widget_class_bind_template_callback (widget_class, sync_cb);
-  gtk_widget_class_bind_template_callback (widget_class, update_cb);
-  gtk_widget_class_bind_template_callback (widget_class, update_amount_tooltip);
   gtk_widget_class_bind_template_callback (widget_class, transactions_clear_cb);
   gtk_widget_class_bind_template_callback (widget_class, browse_flathub_cb);
   gtk_widget_class_bind_template_callback (widget_class, open_search_cb);
@@ -591,6 +567,9 @@ bz_window_class_init (BzWindowClass *klass)
 
   gtk_widget_class_install_action (widget_class, "escape", NULL, action_escape);
   gtk_widget_class_install_action (widget_class, "window.user-data", NULL, action_user_data);
+  gtk_widget_class_install_action (widget_class, "window.open-library", NULL, action_open_library);
+
+  gtk_widget_class_add_binding_action (widget_class, GDK_KEY_d, GDK_CONTROL_MASK, "window.open-library", NULL);
 }
 
 static gboolean
@@ -615,7 +594,7 @@ key_pressed (BzWindow              *self,
 
   visible_child_name = adw_view_stack_get_visible_child_name (self->main_view_stack);
   if (g_strcmp0 (visible_child_name, "installed") == 0)
-    return bz_installed_page_ensure_active (self->installed_page, buf);
+    return bz_library_page_ensure_active (self->library_page, buf);
   else
     {
       adw_view_stack_set_visible_child_name (self->main_view_stack, "search");
@@ -651,22 +630,6 @@ app_busy_changed (BzWindow    *self,
   set_page (self);
 }
 
-static void
-transactions_active_changed (BzWindow             *self,
-                             GParamSpec           *pspec,
-                             BzTransactionManager *manager)
-{
-  check_transactions (self);
-}
-
-static void
-has_transactions_changed (BzWindow             *self,
-                          GParamSpec           *pspec,
-                          BzTransactionManager *manager)
-{
-  check_transactions (self);
-}
-
 static void
 has_inputs_changed (BzWindow          *self,
                     GParamSpec        *pspec,
@@ -676,44 +639,20 @@ has_inputs_changed (BzWindow          *self,
     adw_view_stack_set_visible_child_name (self->main_view_stack, "flathub");
 }
 
-static void
-checking_for_updates_changed (BzWindow    *self,
-                              GParamSpec  *pspec,
-                              BzStateInfo *info)
-{
-  gboolean busy                 = FALSE;
-  gboolean checking_for_updates = FALSE;
-  gboolean has_updates          = FALSE;
-
-  busy                 = bz_state_info_get_busy (info);
-  checking_for_updates = bz_state_info_get_checking_for_updates (info);
-  has_updates          = bz_state_info_get_available_updates (info) != NULL;
-
-  if (!busy && !checking_for_updates)
-    {
-      if (has_updates)
-        {
-          bz_comet_overlay_set_pulse_color (self->comet_overlay, NULL);
-          bz_comet_overlay_pulse_child (
-              self->comet_overlay,
-              GTK_WIDGET (self->update_button));
-        }
-      /* TODO: this can be intrusive when idling checking for updates */
-      // else
-      //   adw_toast_overlay_add_toast (
-      //       self->toasts,
-      //       adw_toast_new_format (_ ("Up to date!")));
-    }
-}
-
 static DexFuture *
 transact_fiber (TransactData *data)
 {
-  g_autoptr (BzWindow) self                           = NULL;
-  g_autoptr (GError) local_error                      = NULL;
-  g_autoptr (BzTransactionDialogResult) dialog_result = NULL;
-  g_autoptr (DexFuture) transact_future               = NULL;
-  g_autofree char *id_dup                             = NULL;
+  g_autoptr (BzWindow) self             = NULL;
+  g_autoptr (GError) local_error        = NULL;
+  g_autoptr (BzEntry) selected_entry    = NULL;
+  g_autoptr (DexFuture) transact_future = NULL;
+  g_autofree char *id_dup               = NULL;
+  BzMainConfig    *config               = NULL;
+  gboolean         delete_user_data     = FALSE;
+  GdkDisplay      *display              = NULL;
+  GdkSeat         *seat                 = NULL;
+  GdkDevice       *keyboard             = NULL;
+  GdkModifierType  modifiers            = GDK_NO_MODIFIER_MASK;
 
   bz_weak_get_or_return_reject (self, data->self);
 
@@ -731,35 +670,134 @@ transact_fiber (TransactData *data)
       bazaar_id = g_application_get_application_id (g_application_get_default ());
       if (g_strcmp0 (id_dup, bazaar_id) == 0)
         {
-          bz_show_error_for_widget (GTK_WIDGET (self), _ ("You can't remove Bazaar from Bazaar!"));
+          bz_show_error_for_widget (GTK_WIDGET (self), _ ("You can't remove Bazaar from Bazaar!"), _ ("You can't remove Bazaar from Bazaar!"));
           return dex_future_new_false ();
         }
     }
 
-  // Show the dialog
-  dialog_result = dex_await_pointer (
-      bz_transaction_dialog_show (
-          GTK_WIDGET (self),
-          data->entry,
-          data->group,
-          data->remove,
-          data->auto_confirm),
-      &local_error);
+  config = bz_state_info_get_main_config (self->state);
+#define RUN_HOOK(_signal)                                              \
+  /* TODO: make reading config less bad */                             \
+  if (config != NULL &&                                                \
+      bz_main_config_get_hooks (config) != NULL)                       \
+    {                                                                  \
+      GListModel *hooks   = NULL;                                      \
+      guint       n_hooks = 0;                                         \
+                                                                       \
+      hooks   = bz_main_config_get_hooks (config);                     \
+      n_hooks = g_list_model_get_n_items (hooks);                      \
+                                                                       \
+      for (guint j = 0; j < n_hooks; j++)                              \
+        {                                                              \
+          g_autoptr (BzHook) hook        = NULL;                       \
+          BzHookSignal       when        = 0;                          \
+          BzHookReturnStatus hook_result = 0;                          \
+                                                                       \
+          hook = g_list_model_get_item (hooks, j);                     \
+          when = bz_hook_get_when (hook);                              \
+                                                                       \
+          if (when == (_signal))                                       \
+            hook_result = dex_await_int (                              \
+                bz_execute_hook (                                      \
+                    hook,                                              \
+                    data->remove                                       \
+                        ? BZ_HOOK_TRANSACTION_TYPE_REMOVAL             \
+                        : BZ_HOOK_TRANSACTION_TYPE_INSTALL,            \
+                    id_dup),                                           \
+                NULL);                                                 \
+                                                                       \
+          if (hook_result == BZ_HOOK_RETURN_STATUS_CONFIRM ||          \
+              hook_result == BZ_HOOK_RETURN_STATUS_STOP)               \
+            break;                                                     \
+          else if (hook_result == BZ_HOOK_RETURN_STATUS_DENY)          \
+            return dex_future_new_reject (                             \
+                BZ_TRANSACTION_MGR_ERROR,                              \
+                BZ_TRANSACTION_MGR_ERROR_CANCELLED_BY_HOOK,            \
+                "The transaction was prevented by a configured hook"); \
+        }                                                              \
+    }
 
-  if (dialog_result == NULL)
-    return dex_future_new_for_error (g_steal_pointer (&local_error));
+  RUN_HOOK (BZ_HOOK_SIGNAL_BEFORE_TRANSACTION);
 
-  if (!dialog_result->confirmed)
-    return dex_future_new_false ();
+  display  = gdk_display_get_default ();
+  seat     = gdk_display_get_default_seat (display);
+  keyboard = gdk_seat_get_keyboard (seat);
+  if (keyboard != NULL)
+    modifiers = gdk_device_get_modifier_state (keyboard);
+
+  if (modifiers & GDK_SHIFT_MASK)
+    /* Holding shift while invoking a transaction skips the dialog and assumes
+       the first valid entry */
+    {
+      if (data->group != NULL)
+        {
+          g_autoptr (GListModel) store = NULL;
+          guint n_items                = 0;
+
+          store = dex_await_object (
+              bz_entry_group_dup_all_into_store (data->group),
+              &local_error);
+          if (store == NULL)
+            return dex_future_new_for_error (g_steal_pointer (&local_error));
+
+          n_items = g_list_model_get_n_items (store);
+          for (guint i = 0; i < n_items; i++)
+            {
+              g_autoptr (BzEntry) entry = NULL;
+              gboolean installed        = FALSE;
+
+              entry     = g_list_model_get_item (store, i);
+              installed = bz_entry_is_installed (entry);
+              if ((data->remove && installed) ||
+                  (!data->remove && !installed))
+                {
+                  selected_entry = g_steal_pointer (&entry);
+                  break;
+                }
+            }
+          if (selected_entry == NULL)
+            return dex_future_new_false ();
+        }
+      else
+        selected_entry = g_object_ref (data->entry);
+    }
+  else
+    {
+      g_autoptr (BzTransactionDialogResult) dialog_result = NULL;
+
+      // Show the dialog
+      dialog_result = dex_await_object (
+          bz_transaction_dialog_show (
+              GTK_WIDGET (self),
+              data->entry,
+              data->group,
+              data->remove,
+              data->auto_confirm),
+          &local_error);
+
+      if (dialog_result == NULL)
+        return dex_future_new_for_error (g_steal_pointer (&local_error));
+      if (!bz_transaction_dialog_result_get_confirmed (dialog_result))
+        return dex_future_new_false ();
+
+      selected_entry = g_object_ref (
+          bz_transaction_dialog_result_get_selected_entry (dialog_result));
+      delete_user_data = bz_transaction_dialog_result_get_delete_user_data (
+          dialog_result);
+    }
 
   // Perform the transaction
-  transact_future = transact (self, dialog_result->selected_entry, data->remove, data->source);
+  transact_future = transact (
+      self,
+      selected_entry,
+      data->remove,
+      data->source);
 
   if (!dex_await (g_steal_pointer (&transact_future), &local_error))
     return dex_future_new_for_error (g_steal_pointer (&local_error));
 
   // Handle user data deletion
-  if (dialog_result->delete_user_data)
+  if (delete_user_data)
     {
       if (data->group != NULL)
         bz_entry_group_reap_user_data (data->group);
@@ -767,34 +805,12 @@ transact_fiber (TransactData *data)
         dex_future_disown (bz_reap_user_data_dex (id_dup));
     }
 
+  RUN_HOOK (BZ_HOOK_SIGNAL_AFTER_TRANSACTION);
+#undef RUN_HOOK
+
   return dex_future_new_true ();
 }
 
-static void
-update_dialog_response (BzTransactionListDialog *dialog,
-                        const char              *response,
-                        BzWindow                *self)
-{
-  if (bz_transaction_list_dialog_was_confirmed (dialog))
-    {
-      GListModel          *updates     = NULL;
-      guint                n_updates   = 0;
-      g_autofree BzEntry **updates_buf = NULL;
-
-      updates     = bz_state_info_get_available_updates (self->state);
-      n_updates   = g_list_model_get_n_items (updates);
-      updates_buf = g_malloc_n (n_updates, sizeof (*updates_buf));
-
-      for (guint i = 0; i < n_updates; i++)
-        updates_buf[i] = g_list_model_get_item (updates, i);
-      update (self, updates_buf, n_updates);
-
-      for (guint i = 0; i < n_updates; i++)
-        g_object_unref (updates_buf[i]);
-      bz_state_info_set_available_updates (self->state, NULL);
-    }
-}
-
 BzWindow *
 bz_window_new (BzStateInfo *state)
 {
@@ -809,22 +825,10 @@ bz_window_new (BzStateInfo *state)
                            "notify::busy",
                            G_CALLBACK (app_busy_changed),
                            window, G_CONNECT_SWAPPED);
-  g_signal_connect_object (state,
-                           "notify::checking-for-updates",
-                           G_CALLBACK (checking_for_updates_changed),
-                           window, G_CONNECT_SWAPPED);
 
   /* these seem unsafe but BzApplication never
    * changes the objects we are connecting to
    */
-  g_signal_connect_object (bz_state_info_get_transaction_manager (state),
-                           "notify::active",
-                           G_CALLBACK (transactions_active_changed),
-                           window, G_CONNECT_SWAPPED);
-  g_signal_connect_object (bz_state_info_get_transaction_manager (state),
-                           "notify::has-transactions",
-                           G_CALLBACK (has_transactions_changed),
-                           window, G_CONNECT_SWAPPED);
   g_signal_connect_object (bz_state_info_get_curated_provider (state),
                            "notify::has-inputs",
                            G_CALLBACK (has_inputs_changed),
@@ -833,7 +837,6 @@ bz_window_new (BzStateInfo *state)
   g_object_notify_by_pspec (G_OBJECT (window), props[PROP_STATE]);
 
   set_page (window);
-  check_transactions (window);
   return window;
 }
 
@@ -845,42 +848,6 @@ bz_window_search (BzWindow   *self,
   search (self, text);
 }
 
-void
-bz_window_toggle_transactions (BzWindow *self)
-{
-  g_return_if_fail (BZ_IS_WINDOW (self));
-  gtk_toggle_button_set_active (
-      self->toggle_transactions,
-      !gtk_toggle_button_get_active (
-          self->toggle_transactions));
-}
-
-void
-bz_window_push_update_dialog (BzWindow *self)
-{
-  GListModel *available_updates = NULL;
-  AdwDialog  *update_dialog     = NULL;
-
-  g_return_if_fail (BZ_IS_WINDOW (self));
-
-  available_updates = bz_state_info_get_available_updates (self->state);
-  g_return_if_fail (available_updates != NULL);
-
-  update_dialog = bz_transaction_list_dialog_new (
-      available_updates,
-      _ ("Updates Are Available"),
-      _ ("The following applications are eligible for updates. Would you like to install them?"),
-      _ ("%d runtimes and/or addons are eligible for updates. Would you like to install them?"),
-      _ ("Additionally, %d runtimes and/or addons will be updated."),
-      _ ("Later"),
-      _ ("Update Now"));
-
-  adw_dialog_set_content_width (update_dialog, 750);
-  g_signal_connect (update_dialog, "response", G_CALLBACK (update_dialog_response), self);
-
-  adw_dialog_present (update_dialog, GTK_WIDGET (self));
-}
-
 void
 bz_window_show_entry (BzWindow *self,
                       BzEntry  *entry)
@@ -936,17 +903,34 @@ bz_window_push_page (BzWindow *self, AdwNavigationPage *page)
     {
       g_signal_connect_swapped (page, "install", G_CALLBACK (install_entry_cb), self);
       g_signal_connect_swapped (page, "remove", G_CALLBACK (remove_installed_cb), self);
-      g_signal_connect_swapped (page, "show-entry", G_CALLBACK (installed_page_show_cb), self);
+      g_signal_connect_swapped (page, "show-entry", G_CALLBACK (select_cb), self);
       g_signal_connect_swapped (page, "bulk-install", G_CALLBACK (bulk_install_cb), self);
-
-      g_object_bind_property (self->split_view, "show-sidebar",
-                              page, "show-sidebar",
-                              G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
     }
 
   adw_navigation_view_push (self->navigation_view, page);
 }
 
+void
+bz_window_bulk_install (BzWindow   *self,
+                        GListModel *groups)
+{
+  g_autoptr (BulkInstallData) data = NULL;
+
+  g_return_if_fail (BZ_IS_WINDOW (self));
+  g_return_if_fail (G_IS_LIST_MODEL (groups));
+
+  data         = bulk_install_data_new ();
+  data->self   = bz_track_weak (self);
+  data->groups = g_object_ref (groups);
+
+  dex_future_disown (dex_scheduler_spawn (
+      dex_scheduler_get_default (),
+      bz_get_dex_stack_size (),
+      (DexFiberFunc) bulk_install_fiber,
+      bulk_install_data_ref (data),
+      bulk_install_data_unref));
+}
+
 BzStateInfo *
 bz_window_get_state_info (BzWindow *self)
 {
@@ -961,8 +945,6 @@ transact (BzWindow  *self,
           GtkWidget *source)
 {
   g_autoptr (BzTransaction) transaction = NULL;
-  GdkPaintable *icon                    = NULL;
-  GtkWidget    *transaction_target      = NULL;
 
   if (remove)
     transaction = bz_transaction_new_full (
@@ -975,44 +957,6 @@ transact (BzWindow  *self,
         NULL, 0,
         NULL, 0);
 
-  if (source == NULL)
-    source = GTK_WIDGET (self->navigation_view);
-
-  if (adw_overlay_split_view_get_show_sidebar (self->split_view))
-    transaction_target = GTK_WIDGET (self->toggle_transactions_sidebar);
-  else
-    transaction_target = GTK_WIDGET (self->toggle_transactions);
-
-  icon = bz_entry_get_icon_paintable (entry);
-  if (icon != NULL)
-    {
-      g_autoptr (BzComet) comet = NULL;
-
-      if (remove)
-        {
-          AdwStyleManager *style_manager = adw_style_manager_get_default ();
-          gboolean         is_dark       = adw_style_manager_get_dark (style_manager);
-          GdkRGBA          destructive_color;
-
-          if (is_dark)
-            destructive_color = (GdkRGBA) { 0.3, 0.2, 0.21, 0.6 };
-          else
-            destructive_color = (GdkRGBA) { 0.95, 0.84, 0.84, 0.6 };
-
-          bz_comet_overlay_set_pulse_color (self->comet_overlay, &destructive_color);
-        }
-      else
-        bz_comet_overlay_set_pulse_color (self->comet_overlay, NULL);
-
-      comet = g_object_new (
-          BZ_TYPE_COMET,
-          "from", remove ? transaction_target : source,
-          "to", remove ? source : transaction_target,
-          "paintable", icon,
-          NULL);
-      bz_comet_overlay_spawn (self->comet_overlay, comet);
-    }
-
   return bz_transaction_manager_add (
       bz_state_info_get_transaction_manager (self->state),
       transaction);
@@ -1052,22 +996,6 @@ try_transact (BzWindow     *self,
       transact_data_ref (data), transact_data_unref));
 }
 
-static void
-update (BzWindow *self,
-        BzEntry **updates,
-        guint     n_updates)
-{
-  g_autoptr (BzTransaction) transaction = NULL;
-
-  transaction = bz_transaction_new_full (
-      NULL, 0,
-      updates, n_updates,
-      NULL, 0);
-  dex_future_disown (bz_transaction_manager_add (
-      bz_state_info_get_transaction_manager (self->state),
-      transaction));
-}
-
 static void
 bulk_install (BzWindow *self,
               BzEntry **installs,
@@ -1103,22 +1031,34 @@ bulk_install_fiber (BulkInstallData *data)
   g_autoptr (BzWindow) self                    = NULL;
   g_autoptr (GError) local_error               = NULL;
   g_autoptr (BzBulkInstallDialogResult) result = NULL;
+  GListModel          *entries                 = NULL;
+  guint                n_installs              = 0;
+  g_autofree BzEntry **installs_buf            = NULL;
 
   bz_weak_get_or_return_reject (self, data->self);
 
-  result = dex_await_pointer (
+  result = dex_await_object (
       bz_bulk_install_dialog_show (GTK_WIDGET (self), data->groups),
       &local_error);
 
   if (result == NULL)
     return dex_future_new_for_error (g_steal_pointer (&local_error));
 
-  if (!result->confirmed)
+  if (!bz_bulk_install_dialog_result_get_confirmed (result))
     return dex_future_new_false ();
 
-  bulk_install (self,
-                (BzEntry **) result->entries->pdata,
-                result->entries->len);
+  entries    = bz_bulk_install_dialog_result_get_entries (result);
+  n_installs = g_list_model_get_n_items (entries);
+  if (n_installs == 0)
+    return dex_future_new_false ();
+
+  installs_buf = g_malloc_n (n_installs, sizeof (*installs_buf));
+  for (guint i = 0; i < n_installs; i++)
+    installs_buf[i] = g_list_model_get_item (entries, i);
+
+  bulk_install (self, installs_buf, n_installs);
+  for (guint i = 0; i < n_installs; i++)
+    g_object_unref (installs_buf[i]);
 
   return dex_future_new_true ();
 }
@@ -1137,43 +1077,6 @@ search (BzWindow   *self,
   gtk_widget_grab_focus (GTK_WIDGET (self->search_widget));
 }
 
-static void
-check_transactions (BzWindow *self)
-{
-  gboolean has_transactions = FALSE;
-  gboolean paused           = FALSE;
-  gboolean active           = FALSE;
-
-  has_transactions = bz_transaction_manager_get_has_transactions (
-      bz_state_info_get_transaction_manager (self->state));
-  adw_view_stack_set_visible_child_name (
-      self->transactions_stack,
-      has_transactions
-          ? "content"
-          : "empty");
-
-  paused = gtk_toggle_button_get_active (self->transactions_pause);
-  active = bz_transaction_manager_get_active (
-      bz_state_info_get_transaction_manager (self->state));
-  if (paused)
-    {
-      gtk_button_set_icon_name (GTK_BUTTON (self->transactions_pause), "media-playback-start-symbolic");
-      gtk_widget_set_tooltip_text (GTK_WIDGET (self->transactions_pause), _ ("Resume Current Tasks"));
-      gtk_widget_add_css_class (GTK_WIDGET (self->transactions_pause), "suggested-action");
-    }
-  else
-    {
-      gtk_button_set_icon_name (GTK_BUTTON (self->transactions_pause), "media-playback-pause-symbolic");
-      gtk_widget_set_tooltip_text (GTK_WIDGET (self->transactions_pause), _ ("Pause Current Tasks"));
-      gtk_widget_remove_css_class (GTK_WIDGET (self->transactions_pause), "suggested-action");
-    }
-
-  if (active)
-    gtk_widget_add_css_class (GTK_WIDGET (self->transactions_stop), "destructive-action");
-  else
-    gtk_widget_remove_css_class (GTK_WIDGET (self->transactions_stop), "destructive-action");
-}
-
 static void
 set_page (BzWindow *self)
 {
